/*
 * Copyright (C) 2009-2010 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "config.h"
#include "amish.h"
#include "amish_cmd_dcf77.h"
#include "dcf77.h"
#include "timer.h"



void amish_cmd_dcf77(amish_cmd_arg *ca) {
	ca->ret.val = 0;
	char *in = ca->input;
	char* out = ca->output;
	uint8_t len = 0;
	uint8_t arg = ca->arg;
	uint8_t state = ((uint32_t8)ca->state).b1;

	if(arg == CMD_DCF_INFO) {
		ca->ret.call_again = 1;
		switch(state) {
		case 0:
			len = sprintf_P(out,PSTR("DCF77 status:\r\n\tin sync: %hhx"), 
				dcf77_info.timer_in_sync);
			break;
		case 1:
			len = sprintf_P(out, PSTR("\ttime valid: %hhx\r\n\tinvalid frames: %hhx"),
				dcf77_info.time_valid, dcf77_info.invalid_frames);
			break;
		case 2:
			len = sprintf_P(out, PSTR("\tavg delay over %u seconds: %u us"),
				mean_edge_counter, timer_ctr_ticks2delay(mean_edge_ticks/mean_edge_counter));
			ca->ret.call_again = 0;
		}
	} else if (arg == CMD_DCF_ASYNC) {
		dcf77_clear_sync();
	} else if (arg == CMD_DCF_DELAY) {
		skip_whitespaces(in);
		if(*in) {
			uint16_t delay = atoi(in);
			dcf77_recv_delay = timer_ctr_delay2ticks(delay);
		}
		len = sprintf_P(out, PSTR("DCF77 reception delay: %u us"), timer_ctr_ticks2delay(dcf77_recv_delay));
	}
	state++;
	ca->state.b1 = state;
	ca->ret.len = len;
}

