/*
 * Copyright (C) 2009-2010 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "config.h"
#include "amish.h"
#include "amish_cmd_timer.h"
#include "timer.h"



void amish_cmd_timer(amish_cmd_arg *ca) {
	ca->ret.val = 0;
	char* out = ca->output;
	uint8_t len = 0;
	uint8_t state = ((uint32_t8)ca->state).b1;
	date_time dt;
	time_t tt;
	uint8_t ds;

	ca->ret.call_again = 1;
	ENTER_CRITICAL_SECTION
		dt = timer_localtime;
		ds = timer_info.decisecond_counter;
		tt = timer_unix_timestamp;
	LEAVE_CRITICAL_SECTION
	switch(state) {
	case 0:
		len = sprintf_P(out,PSTR("Timer status:\r\n\tvalid: %hhx\r\n\tdst: %hhx"), 
			timer_info.valid, dt.dst);
		break;
	case 1:
		len = sprintf_P(out, PSTR("\tdow: %hhx\r\n\tdate: %.4u-%.2hhu-%.2hhu"),
			dt.dow, dt.year, dt.month, dt.day);
		break;
	case 2:
		len = sprintf_P(out, PSTR("\tlocal time: %.2hhu:%.2hhu:%.2hhu.%hhu\r\n\tUnix: %lu"),
			dt.hour, dt.minute, dt.second, ds, tt);
		break;
	case 3:
		ENTER_CRITICAL_SECTION
			tt = timer_uptime;
		LEAVE_CRITICAL_SECTION
		uint8_t s = tt % SECSPERMIN;
		tt /= SECSPERMIN;
		uint8_t m = tt % MINSPERHOUR;
		tt /= MINSPERHOUR;
		uint8_t h = tt % HOURSPERDAY;
		tt /= HOURSPERDAY;
		uint16_t d = tt;
		len = sprintf_P(out, PSTR("\tuptime: %u d %.2hhu:%.2hhu:%.2hhu"),
			d, h, m, s);
		ca->ret.call_again = 0;
	}
	state++;
	ca->state.b1 = state;
	ca->ret.len = len;
}

