/*
 * Copyright (C) 2009-2010 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/pgmspace.h>
#include <stdio.h>
#include "config.h"
#include "amish.h"

/* Includes for command prototypes: */
#include "amish_cmd_uip.h"
#include "amish_cmd_eeprom.h"
#include "amish_cmd_dcf77.h"
#include "amish_cmd_flags.h"
#include "amish_cmd_timer.h"
#include "amish_cmd_main.h"

const char amish_cmd_bootloader_str[] PROGMEM = "bootloader";
const char amish_cmd_dcfinfo_str[] PROGMEM = "dcf77 info";
const char amish_cmd_dcfasync_str[] PROGMEM = "dcf77 clear sync";
const char amish_cmd_dcfdelay_str[] PROGMEM = "dcf77 delay";
const char amish_cmd_eepsave_str[] PROGMEM = "eeprom save";
const char amish_cmd_eepinit_str[] PROGMEM = "eeprom init";
const char amish_cmd_eepinfo_str[] PROGMEM = "eeprom info";
const char amish_cmd_exit_str[] PROGMEM = "exit";
const char amish_cmd_help_str[] PROGMEM = "help";
const char amish_cmd_hwinfo_str[] PROGMEM = "hw info";
const char amish_cmd_ether_str[] PROGMEM = "net ether";
const char amish_cmd_mask_str[] PROGMEM = "net mask";
const char amish_cmd_gw_str[] PROGMEM = "net gw";
const char amish_cmd_ip_str[] PROGMEM = "net ip";
const char amish_cmd_pw_str[] PROGMEM = "password";
const char amish_cmd_reset_str[] PROGMEM = "reset";
const char amish_cmd_showseconds_str[] PROGMEM = "show seconds";
const char amish_cmd_sysinfo_str[] PROGMEM = "sys info";
const char amish_cmd_timerinfo_str[] PROGMEM = "timer info";

const amish_cmd amish_cmds[] PROGMEM = {
	{(char*) amish_cmd_bootloader_str,CMD_SYS_BL,        amish_cmd_sys},
	{(char*) amish_cmd_dcfasync_str,  CMD_DCF_ASYNC,     amish_cmd_dcf77},
	{(char*) amish_cmd_dcfdelay_str,  CMD_DCF_DELAY,     amish_cmd_dcf77},
	{(char*) amish_cmd_dcfinfo_str,   CMD_DCF_INFO,      amish_cmd_dcf77},
	{(char*) amish_cmd_eepinfo_str,   CMD_EEP_INFO,      amish_cmd_eeprom},
	{(char*) amish_cmd_eepinit_str,   CMD_EEP_INIT,      amish_cmd_eeprom},
	{(char*) amish_cmd_eepsave_str,   CMD_EEP_SAVE,      amish_cmd_eeprom},
	{(char*) amish_cmd_exit_str,      CMD_SYS_EXIT,      amish_cmd_sys},
	{(char*) amish_cmd_help_str,      0,                 amish_cmd_help},
	{(char*) amish_cmd_hwinfo_str,    0,                 amish_cmd_main},
	{(char*) amish_cmd_ip_str,        CMD_UIP_GET_IP,    amish_cmd_uip},
	{(char*) amish_cmd_gw_str,        CMD_UIP_GET_GW,    amish_cmd_uip},
	{(char*) amish_cmd_mask_str,      CMD_UIP_GET_MASK,  amish_cmd_uip},
	{(char*) amish_cmd_ether_str,     CMD_UIP_GET_ETHER, amish_cmd_uip},
	{(char*) amish_cmd_pw_str,        CMD_SYS_PW,        amish_cmd_sys},
	{(char*) amish_cmd_reset_str,     CMD_SYS_RESET,     amish_cmd_sys},
	{(char*) amish_cmd_sysinfo_str,   CMD_SYS_INFO,      amish_cmd_sys},
	{(char*) amish_cmd_showseconds_str,CMD_FLAG_SECONDS, amish_cmd_flags},
	{(char*) amish_cmd_timerinfo_str, 0,                 amish_cmd_timer},
	{NULL, 0, NULL} /* Stub to detect the end of the array. Do not delete! */
};
