/*
 * Copyright (C) 2009-2011 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in the
 *		documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _AMISH_UIP_H
#define _AMISH_UIP_H

#include "config.h"
#include "avrutils.h"
#include "amish.h"

#define AMISH_TCP_PORT 23

#define AMISH_UIP_RXBUF 64
#define AMISH_UIP_TXBUF 64

#define AMISH_PW_LEN 8

extern char amish_uip_pw[AMISH_PW_LEN+1];

/*
 * amish_uip_state:
 * State for each AMISH TCP connection.
 * rxbuf, txbuf: Obviously the buffers for incoming/outgoing buffers
 * rxlen, txlen: Length of these buffers
 * reenter_cmd: Set, if the called command needs to send more data.
 * state: see blow;
 * cmd: Pointer to the called command
 */
struct amish_uip_state {
	char rxbuf[AMISH_UIP_RXBUF];
	char txbuf[AMISH_UIP_TXBUF + 2]; /* Reserve a byte for closing \r\n (for sending) or \0 (for application) */
	uint8_t rxlen, txlen;
	amish_cmd_state_t cmd_state;
	amish_cmd* cmd;
	uint8_t reenter_cmd:1;
#define AMISH_UIP_STATE_CONNECTED 0
#define AMISH_UIP_STATE_PW_SENT 1
#define AMISH_UIP_STATE_PROMPT_SENT 2
#define AMISH_UIP_STATE_CMD_PROCESSING 3
	uint8_t state:2;
};

void amish_uip_init(void);

void amish_uip_appcall(void);

#endif
