/*
 * Copyright (C) 2009-2010 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include <util/atomic.h>
#include "config.h"
#include "timer.h"
#include "enc0.h"
#include "uip/uip.h"
#include "avrutils.h"
#include "dcf77.h"
#include "ntpd.h"

void ntpd_init(void) {
	struct uip_udp_conn *conn;
	uip_ipaddr_t addr;
	uip_ipaddr(addr, 255,255,255,255);
	conn = uip_udp_new(&addr, 0);
	if(conn != NULL) {
		uip_udp_bind(conn, HTONS(NTP_PORT));
		DEBUG_PRINT(2, "ntpd: bound\n");
	}
}

void ntpd_process_pkt() {
	if(!uip_newdata())
		return;
	timer_pstamp pstamp;
	timer_get_pstamp(&pstamp);
	struct ntp_pkt *pkt = uip_appdata;
	uint8_t stratum;
	/* Identify ourself as a NTP server. */
	uint8_t mode = PKT_LI_VN_MODE(LEAP_NOWARNING, NTP_VERSION, MODE_SERVER);
	if(dcf77_info.expect_leapsecond)
		mode = PKT_LI_VN_MODE(LEAP_ADDSECOND, NTP_VERSION, MODE_SERVER);
	pkt->li_vn_mode = mode;

	/* Use the reference ID "PTB Modem service" -- it is, kind of ;) */
	if(timer_info.valid) {
		pkt->refid = HTONL(0x50544200);
		stratum = 1;
	} else {
		stratum = 16;
		pkt->refid = 0;
	}
	pkt->stratum = STRATUM_TO_PKT(stratum);

	/* OK, we poll every 16 seconds. Or let we poll the clients every 16s? */
	pkt->ppoll = 4;

	/* Precision is total, until we know how to calculate this. (TODO) */
	pkt->precision = 0xE9;

	/* We have no upstream dispersion or delay here. :-) */
	/*
	 * TODO: Set root delay to calculated reception delay. Possible
	 * if we run on a real crystal oven.
	 */
	pkt->rootdelay = 0;
	pkt->rootdispersion = 0;

	/* client's timestamp -> originate timestamp */
	pkt->org = pkt->xmt;

	/*
	 * Push in our timestamp 
	 * NTP uses a different reference, namely 1900 instead of 1970.
	 * That's why we subtract the seconds of these 70 years.
	 */
	pkt->rec.l_ui = HTONLf(pstamp.seconds + 2208988800);
	pkt->rec.l_uf = HTONLf(pstamp.fraction);

	/* TODO: Change the last sync time to pstamp - pstamp % 60 */
	pkt->reftime = pkt->rec;

	/* Send up the packet's envelope. */
	struct uip_udp_conn conn;
	uip_ipaddr_copy(conn.ripaddr, ((struct uip_tcpip_hdr*)&uip_buf[UIP_LLH_LEN])->srcipaddr);
	conn.rport = ((struct uip_tcpip_hdr*)&uip_buf[UIP_LLH_LEN])->srcport;
	conn.lport = uip_udp_conn->lport;
	uip_udp_conn = &conn;

	/* Fill in the timestamp of the transmit moment at last. */
	timer_get_pstamp(&pstamp);
	pkt->xmt.l_ui = HTONLf(pstamp.seconds + 2208988800);
	pkt->xmt.l_uf = HTONLf(pstamp.fraction);
	
	/* Send the packet. */
	uip_udp_send(sizeof(struct ntp_pkt));

}
