/*
 * Copyright (C) 2009-2010 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _DS_SAA1064_H
#define _DS_SAA1064_H

#include <inttypes.h>
#include <avrutils.h>
#include <util/twi.h>

/*
 * saa1064_init:
 * Initialize a SAA1064 at the given address (A1:A0, range 0-3).
 */
void saa1064_init(uint8_t chipno);

/*
 * saa1064_display:
 * Send the suppied value to the selected chip.
 * The value is an union of two bytes, which represent one segment each.
 * The supplied bytes are ASCII values and are transposed internally.
 * The decimal point is the MSB of each character and is not used for
 * ASCII transformations.
 */
void saa1064_display(uint8_t chipno, uint16_t8 val);

/*
 * saa1064_set_brightness:
 * Set the LED current of the addressed chip.
 * current bit values:
 * current:0:  3 mA
 * current:1:  6 mA
 * current:2: 12 mA
 * Its range is 3 to 21 mA, as you see.
 */
void saa1064_set_brightness(uint8_t chipno, uint8_t current); 

#endif
