/*
 * Copyright (C) 2009-2011 Dennis Schneider
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in the
 *		documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _UIP_CONF_H
#define _UIP_CONF_H

#include "config.h"
#include "uip-appcall.h"

/* Use the maximal ethernet frame size as buffer size: */
#define UIP_CONF_BUFFER_SIZE UIP_MAX_ETH_FRAME_LENGTH

/* AVR byte order: */
#define UIP_CONF_BYTE_ORDER UIP_LITTLE_ENDIAN

/* No IPv6 please (yet) */
#define UIP_CONF_IPV6 0

/* We need UDP support: */
#define UIP_CONF_UDP 1
/* Until now, there is only an NTP service, maybe plus a shell and dhcp client */
#define UIP_CONF_UDP_CONNS 3
#define UIP_CONF_BROADCAST 0
#define UIP_CONF_MULTICAST 0

/* TCP connections: limited to 2, used for shell access */
#define UIP_CONF_MAX_CONNECTIONS 2

/* Disable UIP statistics as we do currently not use them: */
#define UIP_CONF_STATISTICS 0

/* UIP uses different type names: */
typedef uint8_t u8_t;
typedef uint16_t u16_t;
typedef uint32_t u32_t;
typedef uint16_t uip_stats_t;

/* Link layer is ethernet: */
#define UIP_CONF_LLH_LEN 14

/* No NDP needed, IPv4 only */
#define UIP_NEIGHBOR_CONF_ADDRTYPE 0

/* UIP's application is defined in uip-appcall.h */
#define UIP_APPCALL() uip_appcall();
#define UIP_UDP_APPCALL() uip_udp_appcall();

#endif
